==> sdetools-4.8.1.tar.gz: sdetools-4.8.1/sdetools/analysis_integration/mapping.py <==
#!/usr/bin/python
import sys, os
sys.path.append(os.path.dirname(os.path.dirname(os.path.dirname(os.path.abspath(__file__)))))

import csv

from sdetools.extlib.defusedxml import minidom
from sdetools.sdelib.commons import Error


class MappingError(Error):
    pass


class Mapping(object):

    EXPECTED_CSV_HEADERS = ['Task ID', 'Task Title', 'Weakness ID', 'Weakness Title', 'Confidence']

    def __init__(self, weakness_attribute = 'id'):
        self.mapping = {}
        self.weakness_attribute = weakness_attribute

    def load_mapping(self, mapping_file_path):
        file_name, file_ext = os.path.splitext(mapping_file_path)
        file_ext = file_ext[1:]

        if file_ext == 'xml':
            self.load_mapping_from_xml(mapping_file_path)
        elif file_ext == 'csv':
            self.load_mapping_from_csv(mapping_file_path)
        else:
            raise MappingError('%s does not match any supported file type(s): xml, csv' % mapping_file_path)

    def load_mapping_from_xml(self, xml_file):
        self.mapping = {}

        try:
            base = minidom.parse(xml_file)
        except Exception, e:
            raise MappingError("An error occurred opening mapping file '%s': %s" % (xml_file, e))

        for task in base.getElementsByTagName('task'):
            confidence = 'low'
            if task.attributes.has_key('confidence'):
                confidence = task.attributes['confidence'].value

            for weakness in task.getElementsByTagName('weakness'):

                self._register_mapping(
                    weakness.attributes[self.weakness_attribute].value,
                    weakness.attributes['title'].value,
                    task.attributes['id'].value,
                    task.attributes['title'].value,
                    confidence,
                    weakness.attributes['cwe'] if weakness.attributes.has_key('cwe') else None
                )
                
        if not self.mapping:
            raise MappingError("No mapping was found in file '%s'" % xml_file)

    def _register_mapping(self, weakness_id, weakness_title, task_id, task_title, confidence, cwe):
        if weakness_id in self.mapping:
            weakness = self.mapping[weakness_id]
            weakness['tasks'].append({
                'id': task_id,
                'confidence': confidence,
                'title': task_title
            })
        else:
            weakness = {}
            weakness['title'] = weakness_title
            weakness['cwe'] = cwe
            weakness['tasks'] = [{
                'id': task_id,
                'confidence': confidence,
                'title': task_title
            }]

        self.mapping[weakness_id] = weakness

    def load_mapping_from_csv(self, csv_file):

        if isinstance(csv_file, str):
            with open(csv_file, 'rb') as csv_file_object:
                self._load_mapping_from_csv(csv_file_object)
        else:
            self._load_mapping_from_csv(csv_file)

    def _load_mapping_from_csv(self, csv_file):
         self.mapping = {}

         reader = csv.DictReader(csv_file)

         if reader.fieldnames:
             # Check for missing columns
             missing_headers = []
             for header in self.EXPECTED_CSV_HEADERS:
                 if header not in reader.fieldnames:
                     missing_headers.append(header)
         else:
             missing_headers = self.EXPECTED_CSV_HEADERS

         if missing_headers:
             raise MappingError('Missing headers: %s' % ', '.join(missing_headers))
         
         # Gather the mappings
         for row in reader:
             self._register_mapping(
                 row['Weakness %s' % self.weakness_attribute.upper()],
                 row['Weakness Title'],
                 row['Task ID'],
                 row['Task Title'],
                 row['Confidence'],
                 row['CWE'] if 'CWE' in row else None
             )

    def count(self):
        return len(self.mapping.keys())

    def get_weaknesses(self):
        return self.mapping.keys()

    def get_weakness(self, weakness_id):
        if weakness_id in self.mapping:
            return self.mapping[weakness_id]
        else:
            return None

    def contains_weakness(self, weakness_id):
        return self.get_weakness(weakness_id) is not None

    def get_title_for_weakness(self, weakness_id):
        if weakness_id in self.mapping:
            return self.mapping[weakness_id]['title']
        else:
            return None

    def get_task(self, task_id):
        for weakness_id in self.mapping.keys():
            for task in self.mapping[weakness_id]['tasks']:
                if task_id == task['id']:
                    return task
        return None

    def contains_task(self, task_id):
        return self.get_task(task_id) is not None

    def get_tasks(self, weakness_id):
        if self.mapping.has_key(weakness_id):
            return self.mapping[weakness_id]['tasks']
        if '*' in self.mapping:
            return self.mapping['*']['tasks']
        return None

    def get_task_mappings(self, task_id):
        task_mappings = []
        for weakness_id in self.mapping.keys():
            for task in self.mapping[weakness_id]['tasks']:
                if task_id == task['id']:
                    task_mappings.append(self.mapping[weakness_id])
        return task_mappings

    def get_mapped_tasks(self):
        tasks = []
        for weakness_id in self.mapping.keys():
            for task in self.mapping[weakness_id]['tasks']:
                if task['id'] not in tasks:
                    tasks.append(task['id'])

        return tasks

    def to_csv(self, csv_mapping_file):
        with open(csv_mapping_file, 'w') as csv_file:
            writer = csv.DictWriter(csv_file, fieldnames=self.EXPECTED_CSV_HEADERS)
            writer.writerow(dict(zip(self.EXPECTED_CSV_HEADERS, self.EXPECTED_CSV_HEADERS)))

            for weakness_id, mapping in self.mapping.iteritems():
                for task in mapping['tasks']:
                    writer.writerow({
                        'Task ID': task['id'],
                        'Task Title': task['title'],
                        'Weakness ID': weakness_id,
                        'Weakness Title': mapping['title'],
                        'Confidence': task['confidence'],
                    })

def main(argv):

    base = Mapping()
    base.load_mapping_from_xml(argv[1])
    base.to_csv(argv[2])

if __name__ == "__main__":
    main(sys.argv)

==> sdetools-5.0.3.tar.gz: sdetools-5.0.3/sdetools/analysis_integration/mapping.py <==
#!/usr/bin/python
import sys, os
sys.path.append(os.path.dirname(os.path.dirname(os.path.dirname(os.path.abspath(__file__)))))

import csv

from sdetools.extlib.defusedxml import minidom
from sdetools.sdelib.commons import Error


class MappingError(Error):
    pass


class Mapping(object):

    EXPECTED_CSV_HEADERS = ['Task ID', 'Task Title', 'Weakness ID', 'Weakness Title', 'Confidence']

    def __init__(self, weakness_attribute = 'ID'):
        if weakness_attribute not in ['ID', 'Title']:
            raise MappingError('Unexpected Weakness attribute: %s' % weakness_attribute)

        self.mapping = {}
        self.weakness_attribute = weakness_attribute

    def load_mapping(self, mapping_file_path):
        file_name, file_ext = os.path.splitext(mapping_file_path)
        file_ext = file_ext[1:]

        if file_ext == 'xml':
            self.load_mapping_from_xml(mapping_file_path)
        elif file_ext == 'csv':
            self.load_mapping_from_csv(mapping_file_path)
        else:
            raise MappingError('%s does not match any supported file type(s): xml, csv' % mapping_file_path)

    def load_mapping_from_xml(self, xml_file):
        self.mapping = {}

        try:
            base = minidom.parse(xml_file)
        except Exception, e:
            raise MappingError("An error occurred opening mapping file '%s': %s" % (xml_file, e))

        for task in base.getElementsByTagName('task'):
            confidence = 'low'
            if task.attributes.has_key('confidence'):
                confidence = task.attributes['confidence'].value

            for weakness in task.getElementsByTagName('weakness'):

                self._register_mapping(
                    weakness.attributes[self.weakness_attribute.lower()].value,
                    weakness.attributes['title'].value,
                    task.attributes['id'].value,
                    task.attributes['title'].value,
                    confidence,
                    weakness.attributes['cwe'] if weakness.attributes.has_key('cwe') else None
                )

        if not self.mapping:
            raise MappingError("No mapping was found in file '%s'" % xml_file)

    def _register_mapping(self, weakness_id, weakness_title, task_id, task_title, confidence, cwe):
        if weakness_id in self.mapping:
            weakness = self.mapping[weakness_id]
            weakness['tasks'].append({
                'id': task_id,
                'confidence': confidence,
                'title': task_title
            })
        else:
            weakness = {}
            weakness['title'] = weakness_title
            weakness['cwe'] = cwe
            weakness['tasks'] = [{
                'id': task_id,
                'confidence': confidence,
                'title': task_title
            }]

        self.mapping[weakness_id] = weakness

    def load_mapping_from_csv(self, csv_file):

        if isinstance(csv_file, str):
            with open(csv_file, 'rb') as csv_file_object:
                self._load_mapping_from_csv(csv_file_object)
        else:
            self._load_mapping_from_csv(csv_file)

    def _load_mapping_from_csv(self, csv_file):
         self.mapping = {}

         reader = csv.DictReader(csv_file)

         if reader.fieldnames:
             # Check for missing columns
             missing_headers = []
             for header in self.EXPECTED_CSV_HEADERS:
                 if header not in reader.fieldnames:
                     missing_headers.append(header)
         else:
             missing_headers = self.EXPECTED_CSV_HEADERS

         if missing_headers:
             raise MappingError('Missing headers: %s' % ', '.join(missing_headers))

         # Gather the mappings
         for row in reader:
             self._register_mapping(
                 row['Weakness %s' % self.weakness_attribute],
                 row['Weakness Title'],
                 row['Task ID'],
                 row['Task Title'],
                 row['Confidence'],
                 row['CWE'] if 'CWE' in row else None
             )

    def count(self):
        return len(self.mapping.keys())

    def get_weaknesses(self):
        return self.mapping.keys()

    def get_weakness(self, weakness_id):
        if weakness_id in self.mapping:
            return self.mapping[weakness_id]
        else:
            return None

    def contains_weakness(self, weakness_id):
        return self.get_weakness(weakness_id) is not None

    def get_title_for_weakness(self, weakness_id):
        if weakness_id in self.mapping:
            return self.mapping[weakness_id]['title']
        else:
            return None

    def get_task(self, task_id):
        for weakness_id in self.mapping.keys():
            for task in self.mapping[weakness_id]['tasks']:
                if task_id == task['id']:
                    return task
        return None

    def contains_task(self, task_id):
        return self.get_task(task_id) is not None

    def get_tasks(self, weakness_id):
        if self.mapping.has_key(weakness_id):
            return self.mapping[weakness_id]['tasks']
        if '*' in self.mapping:
            return self.mapping['*']['tasks']
        return None

    def get_task_mappings(self, task_id):
        task_mappings = []
        for weakness_id in self.mapping.keys():
            for task in self.mapping[weakness_id]['tasks']:
                if task_id == task['id']:
                    task_mappings.append(self.mapping[weakness_id])
        return task_mappings

    def get_mapped_tasks(self):
        tasks = []
        for weakness_id in self.mapping.keys():
            for task in self.mapping[weakness_id]['tasks']:
                if task['id'] not in tasks:
                    tasks.append(task['id'])

        return tasks

    def to_csv(self, csv_mapping_file):
        with open(csv_mapping_file, 'w') as csv_file:
            writer = csv.DictWriter(csv_file, fieldnames=self.EXPECTED_CSV_HEADERS)
            writer.writerow(dict(zip(self.EXPECTED_CSV_HEADERS, self.EXPECTED_CSV_HEADERS)))

            for weakness_id, mapping in self.mapping.iteritems():
                for task in mapping['tasks']:
                    writer.writerow({
                        'Task ID': task['id'],
                        'Task Title': task['title'],
                        'Weakness ID': weakness_id,
                        'Weakness Title': mapping['title'],
                        'Confidence': task['confidence'],
                    })

def main(argv):

    base = Mapping()
    base.load_mapping_from_xml(argv[1])
    base.to_csv(argv[2])

if __name__ == "__main__":
    main(sys.argv)
